#

from buildz import xf, Base

class Args(Base):
    """
        order: [0, cal]
        args: [(1,), []]
        maps: {
            
        }
    """
    def init(self, conf):
        if type(conf)==str:
            conf = xf.loads(conf)
        self.conf = conf
    def call(self, argx):
        
        pass



from .. import base
from .. import itemz
from ...readz import is_args
class ListMapDeal(base.BaseDeal):
    def init(self, left, right, st, spt):
        self.left = left
        self.right = right
        self.st = st
        self.spt = spt
    def deal(self, obj, conf):
        if not obj.check(is_args=1):
            return None
        list_num = obj.maps("list_num")
        arr = obj.val
        rs = None
        rst = []
        deep = 0
        for i in range(list_num):
            obj = arr[i]
            _val = obj.val
            deep = max(obj.deep+1, deep)
            if i < len(arr)-1:
                _val += conf.s(self.spt)
            rst.append(_val)
        aft_st = conf.s(" ")*conf.get(set=0)
        for i in range(list_num, len(arr), 2):
            obj_k = arr[i]
            obj_v = arr[i+1]
            _val = obj_k.val+conf.s(self.st)+aft_st+obj_v.val
            deep = max(obj_v.deep+1, deep)
            if i+1 < len(arr)-1:
                _val += conf.s(self.spt)
            rst.append(_val)
        if not conf.check(format=1) or conf.get(deep=0)+1>=deep:
            spc = conf.s(" ")*conf.get(prev=0)
            rst = spc.join(rst)
        else:
            rst = [self.fmt(k, conf.get(line=4), conf.get(spc=' ')) for k in rst]
            rst = conf.s("\n").join(rst)
            rst = conf.s("\n")+rst+conf.s("\n")
        rst = conf.s(self.left)+rst+conf.s(self.right)
        return itemz.ShowItem(rst, deep, is_args=1, is_done=1)

pass 



            

                


from . import Base
import os,sys,json,time
class Path(Base):
    def init(self, **maps):
        self.paths = {}
        self.fcs = {}
        self.lasts = {}
        for k, v in maps.items():
            self.set(k, v)
    @staticmethod
    def join(*a):
        return os.path.join(*a)
    @staticmethod
    def rjoin(path, *a):
        if path is None:
            return os.path.join(*a)
        return os.path.join(path, *a)
    @staticmethod
    def rfp(paths, *a, last=-1):
        #print(f"[TESTZ] a: {a}, paths: {paths}")
        #fp = os.path.join(*a)
        for path in paths:
            _fp = Path.rjoin(path, *a)
            if os.path.exists(_fp):
                return _fp
        return Path.rjoin(paths[last], *a)
    def add(self, name, path, index=0):
        self.paths[name].insert(index, path)
    def set(self, name, paths, last = -1, curr = None):
        if type(paths) not in (list, tuple):
            paths = [paths]
        if curr is not None and None not in paths:
            if curr in (0, 'first'):
                paths = [None]+paths
            elif curr in (-1, 'last'):
                paths.append(None)
            else:
                assert False
        self.paths[name] = paths
        self.lasts[name] = last
        def fc(*a):
            return self.rfp(paths, *a, last=last)
        self.fcs[name] = fc
    def __getattr__(self, name):
        return self.fcs[name]
    def call(self, *obj):
        it = obj[0]
        fc = self.join
        if type(it) in (list, tuple):
            assert len(it)==2
            k, fp = it
            fc = self.fcs[k]
        rst = []
        for it in obj:
            if type(it) in (list, tuple):
                assert len(it)==2
                rst.append(it[1])
        return fc(*rst)

pass
